/*
  Chart Alarm

  Version 1.4

  Flashes a lamp and sounds an piezo alarm, essentially to amplify an instrument alarm
  (with an open collector signal, as drawn)

  The code uses the timer library which I find extremely useful. It uses callback routines to pretty much manage the 
  buzzer and lamp on and off times automatically.

  Everything that might require changing is in defines at the top of the code.

  There are a few commented out lines of serial output code I used for debugging. 

  Version 1.4, added a "debounce" time to alarm() because my Simrad NSS9 EVO3 chartplotter seems to go toggle the alarm
  signal before the alarm is really off or canceled.

  Note: there is an inherent flaw in this code associated with the call to millis(). 
    After running the system for approximately 50 days, the timer will wrap around and the timings might go crazy.
    Fix this by checking for wrap if you plan to run your system for more than 50 days at a time or it is configured
    such that it is always on.

  Note: It probably wasn't smart connecting the alarm input to D0, which is also RX as the Nano can't be reprogrammed without
    removing the pullup resistor. It would be better to move it to another digital input pin.

  Rich West, www.twoatsea.com
  My part of this code is in the public domain - use it as you wish. (There may be some code included that is not public domain,
  like Timer.h or the base project I started with.) Happy boating!
*/

// defines
#define CHIRP_TIME  50              //short chirp of piezo used in WARN_2_TIME in milliseconds
#define TIMES_TO_FLASH_1  8         //times to flash lamp only in the first warning functino
#define TIMES_TO_FLASH_2  8         //times to flash lamp and chirp piezo in the second warning function

#define READ_DELAY_TIME 100         //to slow down input read loop

#define WARN_1_ON_TIME  800         //milliseconds to leave lamp on during warning routine
#define WARN_1_OFF_TIME 600         //milliseconds to leave lamp off during warning routine

#define WARN_2_ON_TIME  800         //milliseconds to leave lamp on during warning routine
#define WARN_2_OFF_TIME 600         //milliseconds to leave lamp off during warning routine
#define BUZZ_2_ON_TIME CHIRP_TIME   //time to sound buzzer

#define ALARM_ON_TIME  800          //on time for both led and piezo during final alarm phase in ms
#define ALARM_OFF_TIME 400          //off time for both led and piezo during final alarm phase in ms

#define LED_PIN     10              //output pin for LED warning lamp (with transistor buffer)
#define PIEZO_PIN   11              //output pin for piezo alarm (with transistor buffer)
#define ALARM_IN    0               //pin for alarm signal in from chartplotter

#define ALARM_DEBOUNCE 30           //debouce time in ms for ending alarm()

// calculated defines

#define BUZZ_2_OFF_TIME WARN_2_ON_TIME + WARN_2_OFF_TIME - BUZZ_2_ON_TIME

#define WARN_1_TIME   TIMES_TO_FLASH_1 * (WARN_1_ON_TIME + WARN_1_OFF_TIME)
#define WARN_2_TIME   TIMES_TO_FLASH_2 * (WARN_2_ON_TIME + WARN_2_OFF_TIME)

#define WARN_MILLIS WARN_1_TIME * 1000
#define WARN_2_MILLIS WARN_2_TIME * 1000

#include "Timer.h"

//variable inits
unsigned long targetTime;
Timer t;
int   callbackTO;         //timer callback function sets to 1 when run
int   lampTimerID;
int   buzzerTimerID;

long  lampOnTime;
long  lampOffTime;
long  buzzerOnTime;
long  buzzerOffTime;



// the setup function runs once when you press reset or power the board
void setup() {

  //Serial.begin(9600);
  
    
  // initialize digital pin LED_BUILTIN as an output.
  pinMode(LED_PIN, OUTPUT);
  pinMode(PIEZO_PIN, OUTPUT);
  pinMode(ALARM_IN, INPUT);
}



//************ main loop **********************
// the loop function runs over and over again forever
void loop() {
  /*
  Serial.print("starting");
      Serial.println("");
  Serial.print(WARN_1_TIME);
      Serial.println("");
  Serial.print(WARN_2_TIME);
      Serial.println("");

  */
  
  //wait here as long as ALARM_IN is high
  //set up for open collector signal, low means alarm
  while(digitalRead(ALARM_IN)){
    delay(READ_DELAY_TIME);  //just to prevent too many wasted reads
  }

  //if we're here, alarm was signaled
  warn1();

  //above timed out, is alarm still sounding?
  if(!digitalRead(ALARM_IN)){
    warn2();
  }

  //abouve timed out, is alarm still sounding?
  if(!digitalRead(ALARM_IN)){
    alarm();
  }
 
}

//************ functions ******************

//**********************
void warn1(){
//flash led lamp only for WARN_1_TIME
//light on for WARN_1_ON_TIME, off for WARN_1_OFF_TIME
//lamp automatically updates in timer callback routine
 
  targetTime = millis();        //get initial time - this is how long this warn1() will run
  targetTime += WARN_1_TIME;     //calculate target time
  lampOnTime = WARN_1_ON_TIME;
  lampOffTime = WARN_1_OFF_TIME;
         
  lampTimerID = t.every(lampOnTime, lampCallback);  //start lamp timer
  digitalWrite(LED_PIN, HIGH);                       // turn the LED on   

  //wait for warning time to expire
  //lamp and/or buzzer automatically updates in callback routine
  while(millis() < targetTime){
    t.update();
      
    //check input pin - is timer still sounding?
    if(digitalRead(ALARM_IN)){
      break;
    }  
  }
  
  t.stop(lampTimerID);          //stop timer
  digitalWrite(LED_PIN, LOW);    //lamp off
 }


//**********************
void warn2(){
//flash led lamp and chirp piezo for WARN_2_TIME
//light on for WARN_1_ON_TIME, off for WARN_1_OFF_TIME
//lamp and buzzer automatically update in timer callback routines
 
  targetTime = millis();        //get initial time - this is how long this warn1() will run
  targetTime += WARN_2_TIME;    //calculate target time
  lampOnTime = WARN_2_ON_TIME;
  lampOffTime = WARN_2_OFF_TIME;
  buzzerOnTime = BUZZ_2_ON_TIME;
  buzzerOffTime = BUZZ_2_OFF_TIME;
         
  lampTimerID = t.every(lampOnTime, lampCallback);  //start lamp timer
  digitalWrite(LED_PIN, HIGH);                       // turn the LED on   

  buzzerTimerID = t.every(buzzerOnTime, buzzerCallback);  //start buzzer timer
  digitalWrite(PIEZO_PIN, HIGH);                           // turn the buzzer on   

  //wait for warning time to expire
  //lamp and/or buzzer automatically updates in callback routine
  while(millis() < targetTime){
    t.update();
      
    //check input pin - is timer still sounding?
    if(digitalRead(ALARM_IN)){
      break;
    }  
  }
  
  t.stop(lampTimerID);          //stop timer
  t.stop(buzzerTimerID);        //stop buzzer timer
  digitalWrite(LED_PIN, LOW);    //lamp off
  digitalWrite(PIEZO_PIN, LOW);  //buzzer off

}

//************************
void alarm(){
//full-on major alarm - do it until alarm signal goes low
//sound buzzer and light lamp for ALARM_ON_TIME, off for ALARM_OFF_TIME
//lamp and buzzer automatically update in timer callback routines
 
  unsigned long tTime;

  lampOnTime = ALARM_ON_TIME;
  lampOffTime = ALARM_OFF_TIME;
  buzzerOnTime = ALARM_ON_TIME;
  buzzerOffTime = ALARM_OFF_TIME;
         
  lampTimerID = t.every(lampOnTime, lampCallback);  //start lamp timer
  digitalWrite(LED_PIN, HIGH);                       // turn the LED on   

  buzzerTimerID = t.every(buzzerOnTime, buzzerCallback);  //start buzzer timer
  digitalWrite(PIEZO_PIN, HIGH);                           // turn the buzzer on   

  //loop until alarm signal goes high and stays high for debounce period
  while(true){

    while(!digitalRead(ALARM_IN)){
      t.update();  
    }
   
    //alarm went low, see if it stay slow
    tTime = millis() + ALARM_DEBOUNCE;
    while(digitalRead(ALARM_IN)){
      if(tTime < millis()){
        break;
      }
    }

    //did we break out because debounce time was up?
    if(tTime < millis()){
      break;
    }
    
  }
  
  t.stop(lampTimerID);          //stop timer
  t.stop(buzzerTimerID);        //stop buzzer timer
  digitalWrite(LED_PIN, LOW);    //lamp off
  digitalWrite(PIEZO_PIN, LOW);  //buzzer off

}//end of alarm()

//*************************
void lampCallback(){
  t.stop(lampTimerID);    //cancel timer

  //was lamp on or off
  if(digitalRead(LED_PIN)){
    //lamp was on
    digitalWrite(LED_PIN, LOW);
    //reload callback routine with corrct time
    lampTimerID = t.every(lampOffTime, lampCallback);
    
  }else{
    //lamp was off
    digitalWrite(LED_PIN, HIGH);
    //relaod callback routine with correct time
    lampTimerID = t.every(lampOnTime, lampCallback);
    
  }
  
}

//*************************
void buzzerCallback(){
  t.stop(buzzerTimerID);    //cancel timer

  //was buzzer on or off
  if(digitalRead(PIEZO_PIN)){
    //lamp was on
    digitalWrite(PIEZO_PIN, LOW);
    //reload callback routine with corrct time
    buzzerTimerID = t.every(buzzerOffTime, buzzerCallback);
    
  }else{
    //lamp was off
    digitalWrite(PIEZO_PIN, HIGH);
    //relaod callback routine with correct time
    buzzerTimerID = t.every(buzzerOnTime, buzzerCallback);
    
  }
  
}
